// /home/harris/Projects/FuriusIsoMounter/FuriusIsoMount/FileHash.cs created with MonoDevelop
// User: harris at 18:24 07/01/2008
//
// To change standard headers go to Edit->Preferences->Coding->Standard Headers
//

using System;
using System.Security.Cryptography;
using System.IO;
using System.Text;
using System.Globalization;

namespace FuriusIsoMount
{
	
	
	/// <summary>
    /// Returns a hash value for a file
    /// </summary>
    public sealed class FileHash
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="FileHash"/> class.
        /// </summary>
        private FileHash() { }

        public enum Provider { MD5, SHA1, SHA256, SHA384, SHA512 };

        /// <summary>
        /// Computes the hash.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="hashAlgorithm">The hash algorithm.</param>
        /// <returns></returns>
        public static string ComputeHash(string file, Provider hashAlgorithm)
        {
            Byte[] hashCode;
            using (FileStream fileStream = File.Open(file, FileMode.Open, FileAccess.Read))
            {
                switch (hashAlgorithm)
                {

                    case Provider.MD5:
                        using (MD5CryptoServiceProvider cryptoServiceProvider = new MD5CryptoServiceProvider())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;

                    case Provider.SHA1:
                        using (SHA1CryptoServiceProvider cryptoServiceProvider = new SHA1CryptoServiceProvider())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                    case Provider.SHA256:
                        using (SHA256Managed cryptoServiceProvider = new SHA256Managed())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                    case Provider.SHA384:
                        using (SHA384Managed cryptoServiceProvider = new SHA384Managed())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                    case Provider.SHA512:
                        using (SHA512Managed cryptoServiceProvider = new SHA512Managed())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                    default:
                        using (MD5CryptoServiceProvider cryptoServiceProvider = new MD5CryptoServiceProvider())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                }
            }
            StringBuilder stringBuilder = new StringBuilder();
            for (int i = 0; i < hashCode.Length; i++)
            {
                stringBuilder.Append(hashCode[i].ToString("x2", CultureInfo.InvariantCulture));
            }

            return stringBuilder.ToString();
        }
    }
}
