// /home/harris/Projects/FuriusIsoMounter/FuriusIsoMount/ImageAction.cs created with MonoDevelop
// User: harris at 20:58 07/01/2008
//
// To change standard headers go to Edit->Preferences->Coding->Standard Headers
//

using System;
using System.IO;
using FuriusIsoMount;
using System.Diagnostics;
using Gtk;
using System.Threading;

namespace FuriusIsoMount
{
	
	/// <summary>
	/// Mounts an Image
	/// </summary>
	public static class ImageAction
	{
		
		//public enum ImageType {ISO, IMG};
		
		/// <summary>
	    /// Mounts an Image
	    /// </summary>
		/// <param name="imageFile">The filename.</param>
        /// <param name="treeStore">The TreeStore used to store the current mounts</param>
		public static void Mount(string imageFile, TreeStore treeStore)
		{
			FileInfo file = new FileInfo(imageFile);
			//Replace any characters that could screw us up.
			string mountLocation = @"/media/" + file.Name.Replace(' ', '_');
			mountLocation = mountLocation.Replace('.', '_');
			try
			{
				// If an image is already mounted this will throw an IOException
				if (Directory.Exists(mountLocation))
				{
					Directory.Delete(mountLocation);
				}
				Directory.CreateDirectory(mountLocation);
				// Mount the image
				Process.Start("mount -t iso9660 -o loop '" + imageFile + "' '"+ mountLocation +"'");
				
				//Write an entry with instructions on how to remove manually if needed
				Log.Write(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), 
				                       "FuriusMountLog.log"), 
				          string.Format("{0} successfully mounted @ {1}. If required, " + 
				                        "run the following commands with root privileges to remove:\n" + 
				                        "  :umount {1}\n  :rmdir {1}", imageFile, mountLocation));
				
				// Add mount point and image to treeStore
				treeStore.AppendValues (mountLocation, imageFile);
				
				
			}
			catch (UnauthorizedAccessException exception)
			{
				Log.Write(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), 
				                       "FuriusMountLog.log"), exception.Message);
				throw;
			}
			catch (IOException exception)
			{
				Log.Write(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), 
				                       "FuriusMountLog.log"), exception.Message);
				throw;
			}
		}
		
		/// <summary>
	    /// Unmounts an Image
	    /// </summary>
		/// <param name="mountPoint">The mount point.</param>
        /// <param name="treeStore">The TreeStore used to store the current mounts</param>
		/// <param name="iter">The TreeIter to remove from the TreeStore</param>
		public static void Unmount(string mountPoint, TreeStore treeStore, TreeIter iter)
		{
			try
			{
				// If the directory doesn't exist, it may have been
				// unmounted externally so just remove the TreeIter
				// from the TreeStore
				if (Directory.Exists(mountPoint))
				{
					// Start as a named process so we can ensure it
					// has completed before trying to remove the directory
					Process process = Process.Start("umount '"+ mountPoint +"'");			
					while (!process.HasExited)
			        {
			            Thread.Sleep(500); // sleep  			
			        }
					Thread.Sleep(500); // Patience my pretty!
					// When unmounting has completed delete the directory
					Directory.Delete(mountPoint);
					Log.Write(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), 
				                       "FuriusMountLog.log"), 
				          string.Format("{0} successfully unmounted and deleted.", mountPoint));
				}
				// Remove mount point info from the TreeStore
				treeStore.Remove(ref iter);
			}
			catch (UnauthorizedAccessException exception)
			{
				Log.Write(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), 
				                       "FuriusMountLog.log"), exception.Message);
				throw;
			}
			catch (IOException exception)
			{
				Log.Write(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), 
				                       "FuriusMountLog.log"), exception.Message);
				throw;
			}
		}
		
		/// <summary>
	    /// Opens Nautilus at the selected mount point
	    /// </summary>
		/// <param name="mountPoint">The mount point.</param>
		public static void Browse(string mountPoint)
		{
			 Process.Start("nautilus --browser '"+ mountPoint +"'");
			 Log.Write(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), 
				                       "FuriusMountLog.log"), 
				          string.Format("Command attempted: nautilus --browser '{0}'", mountPoint));
		}
		
		/// <summary>
	    /// Starts the Nautilus CD burning wizard
	    /// </summary>
		/// <param name="imageFile">The Image file to burn</param>
		public static void Burn(string imageFile)
		{
			Process.Start("nautilus-cd-burner --source-iso='"+ imageFile +"'");
			Log.Write(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), 
				                       "FuriusMountLog.log"), 
				          string.Format("Command attempted: nautilus-cd-burner --source-iso='{0}'", imageFile));
		}
	}
}
