// /home/harris/Projects/FuriusIsoMounter/FuriusIsoMount/FuriusIsoMount/MainWindow.cs created with MonoDevelop
// User: harris at 17:58 07/01/2008
//
// To change standard headers go to Edit->Preferences->Coding->Standard Headers
//
using System;
using Gtk;
using System.IO;
using FuriusIsoMount;
using System.Threading;
using System.Diagnostics;
using System.Collections;
using System.Xml.Serialization;

public partial class MainWindow: Gtk.Window
{	
	private TreeStore isoListStore;
	private TreeSelection selection;
	private TreeIter iter;
	
	private Tooltips tooltips = new Tooltips();	
	private Thread computeHashThread;

		
	/// <summary>
    /// Initializes a new instance of the <see cref="MainWindow"/> class.
    /// </summary>
	public MainWindow (): base (Gtk.WindowType.Toplevel)
	{
		Log.Write(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log", "Application started...");
		Build ();
		CreateTreeviewColumns();
		//Modify Font size
		this.labelSelectedImage.ModifyFont(Pango.FontDescription.FromString("8"));
		this.labelSelectedMountPoint.ModifyFont(Pango.FontDescription.FromString("8"));
		this.labelHash.ModifyFont(Pango.FontDescription.FromString("8"));
		//Set some tooltips
		tooltips.SetTip(this.treeviewMountedImages, "Mounted Images and Mount Points are listed here.\nDouble clicking will open the selected mount point in Nautilus File Manager.", null);
		tooltips.SetTip(this.buttonBrowse, "Browse for an Image file\n(ISO or IMG files supported).", null);
		tooltips.SetTip(this.buttonMount, "Mount selected Image file.", null);
		tooltips.SetTip(this.buttonUnmount, "Unmount selected Mount Point.", null);
		tooltips.SetTip(this.buttonChecksum, "Calculate Checksum.", null);
		tooltips.SetTip(this.buttonBurn, "Burn Image to optical disk.", null);
	}
	
		
	/// <summary>
	/// Creates the TreeView Columns and sets the datasource (TreeStore)
	/// </summary>
	private void CreateTreeviewColumns()
	{	 
		this.treeviewMountedImages.AppendColumn ("Mount Point", new Gtk.CellRendererText (), "text", 0);
		this.treeviewMountedImages.AppendColumn ("Image File", new Gtk.CellRendererText (), "text", 1);
		this.isoListStore = new Gtk.TreeStore (typeof (string), typeof (string));		
		this.treeviewMountedImages.Columns[0].SortColumnId = 0;
		this.treeviewMountedImages.Columns[1].SortColumnId = 1;
		this.treeviewMountedImages.Model = this.isoListStore;
				
	}
	
	/// <summary>
    /// Handles the DeleteEvent event of the MainWindow control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="a">The <see cref="DeleteEventArgs"/> instance containing the event data.</param>
	protected void OnDeleteEvent (object sender, DeleteEventArgs a)
	{	

		this.treeviewMountedImages.Model.Foreach(CleanUp);
				
		Application.Quit ();
		a.RetVal = true;
		Log.Write(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log", "Application closed!");
		
	}
	
	/// <summary>
    /// Iterates over each TreeIter and request confirmation to unmount any mounted images
    /// </summary>
    /// <param name="model">TreeModel</param>
    /// <param name="path">TreePath</param>
	/// <param name="iter">TreeIter</param>
	/// <remarks>
	/// The three parameters: model, path, and iter, must be included when
    /// you define a function called by TreeModel.Foreach, but you don't have to specifically pass them
    /// because they will get pass automaticallly.
	/// If you return [true] Iteration cancels
	/// </remarks>
	/// <returns>[false]</returns>
	private bool CleanUp(TreeModel model, TreePath path, TreeIter iter)
	{
		
		if (MessageBox.Show(this, string.Format("{0} is still mounted.\nDo you wish to unmount it before exiting?", model.GetValue(iter, 1).ToString()), MessageType.Question, ButtonsType.YesNo) == ResponseType.Yes)
		{
			try // and unmount the image
			{
				ImageAction.Unmount(model.GetValue(iter, 0).ToString(), this.isoListStore, this.iter);
			}
			catch (Exception exception)
			{
				Log.Write(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log", exception.Message);
			}
		}
		
		return false;
	}
		
	/// <summary>
    /// Handles the Click event of the buttonBrowse control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnButtonBrowseClicked (object sender, System.EventArgs e)
	{
		SelectAndVerifyImage();
	}
	
	/// <summary>
    /// Selects and Verifies the Image file by calling
	/// SelectImage and VerifyImage methods
    /// </summary>
	private void SelectAndVerifyImage()
	{		
		if(!SelectImage())
		{
			return;
		}
		VerifyImage();		
	}
	
	/// <summary>
    /// Handles the Click event of the buttonBrowse control.
    /// </summary>
	/// <returns>[true] if a file is selected</returns>
	private bool SelectImage()
	{
		// Create a new FileSelection Dialog
		FileSelection fileSelection = new FileSelection ("Open");
		if ( (Gtk.ResponseType) fileSelection.Run () != Gtk.ResponseType.Ok) 
		{
			fileSelection.Hide ();
			return false;
		}
		this.entryImage.Text = fileSelection.Filename;
		fileSelection.Hide ();
		return true;
	}

	/// <summary>
    /// Verifies that an Image (.iso or .img) file has been selected.
    /// </summary>
	private void VerifyImage()
	{
		string file = this.entryImage.Text;
		// Check the file exists
		if (!File.Exists(file))
		{
			MessageBox.Show(this, "Unable to find file. \nPlease check source exists.", MessageType.Error);
			this.labelSelectedImage.Text = "No Image Selected";
			this.buttonMount.Sensitive = false;
			this.buttonChecksum.Sensitive = false;
			this.buttonBurn.Sensitive = false;
			return;
		}
		//Check the files an Image
		if (file.ToLower().EndsWith(".iso") || file.ToLower().EndsWith(".img"))
		{
			this.labelSelectedImage.Text = file;
			this.buttonMount.Sensitive = true;
			this.buttonChecksum.Sensitive = true;
			this.buttonBurn.Sensitive = true;
			return;
		}
		
		MessageBox.Show(this, "File does not appear to be an ISO. \nPlease check source.", MessageType.Error);
		this.labelSelectedImage.Text = "No Image Selected";
		this.buttonMount.Sensitive = false;
		this.buttonChecksum.Sensitive = false;
		this.buttonBurn.Sensitive = false;
				
	}

	/// <summary>
    /// Handles the Click event of the buttonMount control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnButtonMountClicked (object sender, System.EventArgs e)
	{
		try // and mount the Image
		{
			ImageAction.Mount(this.labelSelectedImage.Text, this.isoListStore);
		}
		catch (UnauthorizedAccessException exception)
		{
			MessageBox.Show(this, exception.Message +
				                "\nYou do not appear to have root access!\nPlease restart application as root.", 
				                MessageType.Error);
		}
		catch (IOException exception)
		{
			MessageBox.Show(this, exception.Message +
				                "\nPossible cause: Image already mounted.\n If so, please unmount before remounting.", 
				                MessageType.Error);
		}
	}

	/// <summary>
    /// Handles the CursorChanged event of the treeviewMountedImages control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnTreeviewMountedImagesCursorChanged (object sender, System.EventArgs e)
	{
		TreeModel model;
		// Get the selected TreeModel and TreeIter
		this.selection = (sender as TreeView).Selection;
		selection.GetSelected(out model, out iter);
		//Get the value of the first item (0 based index) of the selected TreeIter
		this.labelSelectedMountPoint.Text = this.treeviewMountedImages.Model.GetValue(iter, 0).ToString();
		// Allow user to unmount
		this.buttonUnmount.Sensitive = true;
		
	}

	/// <summary>
    /// Handles the Click event of the buttonUnmount control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnButtonUnmountClicked (object sender, System.EventArgs e)
	{
		try // and unmount the image
		{
			ImageAction.Unmount(this.labelSelectedMountPoint.Text, this.isoListStore, this.iter);
			//This will delete the current TreeIter so no mount points are selected.
			this.labelSelectedMountPoint.Text = "No Mount Point Selected";
			this.buttonUnmount.Sensitive = false;
		}
		catch (UnauthorizedAccessException exception)
		{
			MessageBox.Show(this, exception.Message +
				                "\nYou do not appear to have root access!\nPlease restart application as root.", 
				                MessageType.Error);
		}
		catch (IOException exception)
		{
			MessageBox.Show(this, exception.Message +
				                "\nPossible cause: Folder deletion attempted before unmount process complete.\n If so, please unmount try again.", 
				                MessageType.Error);
		}
	}
	
	

	/// <summary>
    /// Handles the Click event of the buttonChecksum control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnButtonChecksumClicked (object sender, System.EventArgs e)
	{	
		
		if (this.buttonChecksum.Label == "_Checksum")
		{
			this.computeHashThread = new Thread (new ThreadStart (ComputeChecksum));
			this.computeHashThread.Start ();
			Log.Write(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log", "Checksum generation started...");
			this.labelHash.Text = "Calculating Checksum. Please wait...";
			this.buttonChecksum.Label = "_Cancel";
		}
		else
		{
			if (this.computeHashThread.IsAlive)
			{
				this.computeHashThread.Abort(); // Not the ideal way to stop a thread!
				this.labelHash.Text = "No Checksum Generated";
			}
			this.buttonChecksum.Label = "_Checksum";
			Log.Write(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log", "Checksum generation aborted!");
		}
	}
	
	/// <summary>
    /// Computes the selected Checksum
    /// </summary>
	/// <remarks>
	/// Gtk is not thread safe, so uses Gtk.Application.Invoke
	/// to make code run on the main loop thread.
	/// </remarks>
	private void ComputeChecksum ()
    {
        string hashValue;
		if (this.radiobuttonMd5.Active)
		{
			hashValue = FileHash.ComputeHash(this.entryImage.Text, FileHash.Provider.MD5);
		}
		else
		{
			hashValue = FileHash.ComputeHash(this.entryImage.Text, FileHash.Provider.SHA1);
		}
		
		Gtk.Application.Invoke (delegate {
            this.labelHash.Text = hashValue;
			this.buttonChecksum.Label = "_Checksum";
			Log.Write(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log", "Checksum generation complete");
        });
    }

	/// <summary>
    /// Handles the Click event of the buttonBurn control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnButtonBurnClicked (object sender, System.EventArgs e)
	{
		ImageAction.Burn(this.entryImage.Text);
	}

	/// <summary>
    /// Handles the Click event of the buttonAbout control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnButtonAboutClicked (object sender, System.EventArgs e)
	{
		Process.Start("http://www.marcus-furius.com/?page_id=14");
	}

	/// <summary>
    /// Handles the Click event of the buttonEventLog control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnButtonEventLogClicked (object sender, System.EventArgs e)
	{
		if (File.Exists(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log"))
		{
			Process.Start(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log");
		}
		else
		{
			MessageBox.Show(this, "FuriusMountLog.log not found.", MessageType.Info); 
		}
	}

	/// <summary>
    /// Handles the Click event of the buttonDeleteLog control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
	protected virtual void OnButtonDeleteLogClicked (object sender, System.EventArgs e)
	{
		if (File.Exists(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log"))
		{
			if (MessageBox.Show(this, 
		        "Are you sure you wish to delete the log?", 
		        MessageType.Warning, ButtonsType.YesNo) == ResponseType.Yes)
			{
				File.Delete(Environment.GetFolderPath(Environment.SpecialFolder.Personal) +  
				                       "/FuriusMountLog.log");
			}
		}
	}

	/// <summary>
    /// Handles the RowActivated event of the treeviewMountedImages control.
    /// </summary>
    /// <param name="o">The object of the event.</param>
    /// <param name="args">The <see cref="Gtk.RowActivatedArgs"/> instance containing the event data.</param>
	protected virtual void OnTreeviewMountedImagesRowActivated (object o, Gtk.RowActivatedArgs args)
	{
		if (MessageBox.Show(this, 
		        "If you started this application as root, please be aware that this will open Nautilus File Manager with root privileges!\nAre you sure you wish to continue?", 
		        MessageType.Warning, ButtonsType.YesNo) == ResponseType.Yes)
		{
			ImageAction.Browse(this.labelSelectedMountPoint.Text);
		}
	}
	
}